<?php

namespace FiloBlu\CategoryAdminFilter\Ui\Component\Listing\Column;

use Magento\Catalog\Api\CategoryRepositoryInterface;
use Magento\Catalog\Model\ProductCategoryList;
use Magento\Framework\View\Element\UiComponent\ContextInterface;
use Magento\Framework\View\Element\UiComponentFactory;
use Magento\Ui\Component\Listing\Columns\Column;

class Category extends Column
{
    /**
     * @var ProductCategoryList
     */
    private $productCategory;

    /**
     * @var CategoryRepositoryInterface
     */
    private $categoryRepository;

    /**
     * @param ContextInterface $context
     * @param UiComponentFactory $uiComponentFactory
     * @param array $components
     * @param array $data
     * @param ProductCategoryList $productCategory
     * @param CategoryRepositoryInterface $categoryRepository
     */

    public function __construct(
        ContextInterface $context,
        UiComponentFactory $uiComponentFactory,
        ProductCategoryList $productCategory,
        CategoryRepositoryInterface $categoryRepository,
        array $components = [],
        array $data = []
    ) {
        $this->productCategory = $productCategory;
        $this->categoryRepository = $categoryRepository;
        parent::__construct($context, $uiComponentFactory, $components, $data);
    }

    /**
     * Prepare data for the category column
     * @param  array  $dataSource
     * @return array
     */
    public function prepareDataSource(array $dataSource)
    {
        $fieldName = $this->getData('name');
        if (isset($dataSource['data']['items'])) {
            foreach ($dataSource['data']['items'] as &$item) {
                $productId = $item['entity_id'];
                $categoryIds = $this->getCategoryIds($productId);
                $categories = [];
                if (count($categoryIds)) {
                    foreach ($categoryIds as $categoryId) {
                        $categoryData = $this->categoryRepository->get($categoryId);
                        $categories[] = $categoryData->getName();
                    }
                }
                $item[$fieldName] = implode(', ', $categories);
            }
        }
        return $dataSource;
    }

    /**
     * Retrieve all the category ids
     *
     * @param integer $productId
     * @return array
     */
    private function getCategoryIds($productId)
    {
        $categoryIds = $this->productCategory->getCategoryIds($productId);
        $_categoryIds = [];
        if ($categoryIds) {
            $_categoryIds = array_unique($categoryIds);
        }
        return $_categoryIds;
    }
}
